-- ****************************************************************************************************
-- Change Log:
-- Version 3.0.221225
--
-- 2022-12-25
-- Merry Christmas and Happy Holidays!
-- Added Dragonriding category and support for Dragonriding mounts where they have been enabled.
--
-- 2022-11-15
-- Updated interface number for 10.0.2
--
-- 2022-10-27
-- Updated interface number for 10.0.0
-- Removed an unnecessary line of XML that now causes a LUA error.
--
-- 2022-06-07
-- Updated interface number for 9.2.5 (sorry for never updating to 9.2.0 - life got the better of me
-- for a while there...)
--
-- 2021-11-02
-- Updated interface number for 9.1.5
--
-- 2021-07-02
-- Updated version and interface numbers for 9.1
--
-- 2021-03-11
-- Updated version and interface numbers for 9.0.5
-- Updated "Resolve Area" logic to override group status in areas with specific mount categories
--
-- 2021-01-21
-- Added debug option to assist with diagnosis of issues in logic.
-- Added IsInMaw() check to the ResolveFlying() logic to allow Maw category mounts to be used when
--   in the starting/intro Shadowlands area
--
-- 2021-01-14
-- Updated control key override to summon a favorite mount instead of summoning a selected flying mount
-- Added logic to prevent a blank list when trying to add mounts to a category.
--
-- 2020-12-30
-- Added saved variables for layout state by character and account.
-- Added button in Interface > Addons to rotate layout states.
--
-- 2020-12-29
-- Added Interface > Addons page for Khes'yc Mount - currently only holds a button until I can build
--   the minimalist UI and tie it to a button
-- Removed reference to Khesyc_Mount.lua from Khesyc_Mount.toc since it's called in Khesyc_Mount.xml 
--   and the two refrences were causing everything to run in duplicate
-- Updated version control on the XML and in the LUA to pull the version number from the TOC
-- Added IsJointMount to join two mounts by ID across factions - this allows me to more easily join
--   additional pairs of mounts together in the future, like the Ironclad Frostclaw and the Bloodflank
--   Charger.
-- Added /kmlayout command to switch main UI layout between "full" and "min".
-- Moved speedometer update event to pitch button for minimal layout option.
--
-- 2020-11-23
-- Added category to use Maw mounts into XML category display
-- 
-- 2020-11-18
-- Updated interface support version
-- Added logic to correct Draenor Pathfinding not returning correctly on new characters
--
-- 2020-10-11
-- Added Ahn'Qiraj category and support for Maw category in Shadowlands
-- Updated logic to handle missing category lists
-- Added event handling logic to reload mount list when a new mount is added
--
-- 2020-09-23
-- Updated ToonCanMount and ToonCanFlyArea logic to assume Legacy - Artisan Riding skill if no mount skill
--   at all is known at or above level 27
--
-- 2020-09-15
-- New minor build.
-- Resolved some technical debt with repeated XML elements.
-- Added model previewer and mount tooltip to mount list.
-- Known issue: Unable to identify riding skill for toons with Legacy: Artisan Riding level
--
-- 2020-09-13
-- Updated the Settings frame to be movable.
-- Updated frame backdrops to work with Shadowlands changes.
--
-- -2020-09-12
-- Updated swimming mount logic to use a timer from the last time IsSubmerged() was not "true"
--
-- 2020-09-11
-- Resolved some bugs in the previous build around changing Option Types while adding mounts.
-- Added new tips screen to help users know how the addon works.
-- Added version comparison to automatically open tips on new version updates.
--
-- 2020-09-09
-- Version number update!
-- Updated mount variables to be account-wide except ground and flying.
-- Reworking settings to display more than one mount in a group at once.
-- Reworking summon logic to randomly select a mount from the group (as appropriate).
--
-- 2020-08-04
-- Updated mount select list to be alphabetically sorted instead of numerically sorted by ID.
-- Also changed settings button to close settings UI if it's already open.
--
-- 2020-04-01
-- Updated logic on swimming mounts to default Vashj'ir Seahorse in Vashj'ir if it's available.
-- Also added pitch lock feature to help keep flying and swimming mounts locked on their pitch.
--
-- 2019-03-07
-- Added ScrollFrame children to Settings Frame for select mounts, ground mounts, flying mounts,
--   swimming mounts, and water mounts.
-- Added ScrollFrame children for ground and flying passenger mounts.
-- Changed settings frame layout to accomodate additional frames.
-- 
-- 2019-03-16
-- Added date as minor build number instead of static "0" as yyyymmdd.
-- Added UI Frame for the Settings and wired up settings button and close button to properly display
--   and hide the settings frame.
-- Still need to figure out how to build a List Box in WoW Lua/XML.
--
-- Version 1.0.0
-- Initial build. Adds UI Frame with button to summon mount from static list.
-- Limited distribution for personal use only.
-- ****************************************************************************************************

SLASH_KHESYCMOUNT1 = "/km";
SlashCmdList["KHESYCMOUNT"] = function(msg)
   KhesycMountSummon();
end

SLASH_KHESYCMOUNTSETTINGS1 = "/kmsettings";
SlashCmdList["KHESYCMOUNTSETTINGS"] = function(msg)
	KhesycMountSettingsOpen();
end

SLASH_KMINFO1 = "/kminfo";
SlashCmdList["KMINFO"] = function(msg, editbox)
	local i,m = GetMouseFocus().index;
	if i then
		m = {C_MountJournal.GetDisplayedMountInfo(GetMouseFocus().index)}
		print(m[1],"is mountID",m[12]);
	end
end

SLASH_KMTIPS1 = "/kmtips";
SlashCmdList["KMTIPS"] = function(msg)
	if km_Tips:IsVisible() then
		km_Tips:Hide();
	else
		km_Tips:Show();
	end
end

SLASH_KMRELOAD1 = "/kmreload";
SlashCmdList["KMRELOAD"] = function(msg)
	KhesycMountReload();
end

SLASH_KMLAYOUT1 = "/kmlayout";
SlashCmdList["KMLAYOUT"] = function(msg)
	KhesycMountSwitchLayout(msg);
end

local km_debug = false;
SLASH_KMDEBUG1 = "/kmdebug";
SlashCmdList["KMDEBUG"] = function(msg)
	if (km_debug) then
		print("Khes'yc Mount - Debug Mode: Disabled");
		km_debug = false;
	else
		print("Khes'yc Mount - Debug Mode: Enabled");
		km_debug = true;
	end
end

--declarations
local addon = ...;
local addonTitle = GetAddOnMetadata(addon, "Title");
local addonVersion = GetAddOnMetadata(addon, "Version");
local ids = {};
local names = {};
local nameKey = {};
local maxInd = 1;
local listInd = 1;
local km_list = {};
local hasVashjir = 0;
local lastDryTime = GetTime();
local typeOption = "flying";
local effect = "mount";

--GLOBALS
local _G = _G;
local InterfaceOptions_AddCategory = _G.InterfaceOptions_AddCategory;

--Create the Blizzard addon option frame
local panel = CreateFrame("Frame", addonTitle .. "BlizzOptions");
panel.name = addonTitle;
InterfaceOptions_AddCategory(panel);

local fs = panel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
fs:SetPoint("TOPLEFT", 10, -15);
fs:SetJustifyH("LEFT");
fs:SetJustifyV("TOP");
fs:SetText(addonTitle);

local st = panel:CreateFontString(nil, "OVERLAY", "SystemFont_Outline");
st:SetPoint("TOPLEFT", 10, -48);
st:SetPoint("BOTTOMRIGHT", panel, "TOPRIGHT", 10, -80);
st:SetJustifyH("LEFT");
st:SetJustifyV("TOP");
st:SetText("Please consider supporting this and our other projects at www.khesyc.com via Patreon at www.patreon.com/khesyc or Venmo at @Khesyc.");

local button = CreateFrame("Button", nil, panel, "UIPanelButtonTemplate");
button:SetText("Open Settings");
button:SetWidth(128);
button:SetPoint("TOPLEFT", 10, -85);
button:SetScript('OnClick', function()
	while CloseWindows() do end;
	KhesycMountSettingsOpen();
end);

local lblLayout = panel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
lblLayout:SetPoint("TOPLEFT", 10, -120);
lblLayout:SetJustifyH("LEFT");
lblLayout:SetJustifyV("TOP");
lblLayout:SetText("Layout Style");

local btnLayout = CreateFrame("Button", "km_btnLayout", panel, "UIPanelButtonTemplate");
btnLayout:SetWidth(100);
btnLayout:SetPoint("TOPLEFT", 30, -135);
btnLayout:SetScript('OnClick', function()
	if (km_indLayout == "full") then
		KhesycMountSwitchLayout("min");
	elseif (km_indLayout == "min") then
		KhesycMountSwitchLayout("full");
	end
end);

--bindings labels
BINDING_HEADER_TITLE = "Khes'yc Mount";
BINDING_NAME_SUMMON = "Mount";
BINDING_NAME_VENDOR = "Vendor Mount";
BINDING_NAME_SWIM = "Swimming Mount";
BINDING_NAME_FLYING = "Override Mount";

--Create event handler frame
local km_Event = CreateFrame("frame", "km_Event");
km_Event:RegisterEvent("VARIABLES_LOADED");
km_Event:RegisterEvent("MOUNT_JOURNAL_USABILITY_CHANGED");
km_Event:RegisterEvent("NEW_MOUNT_ADDED");

km_Event:SetScript("OnEvent", function(self, event, ...)
	if (event == "VARIABLES_LOADED") then
		KhesycMountInit();
		KhesycMountBuild();
		KhesycMountCheckVersion();
		km_Tips.km_Version:SetText("v. " .. addonVersion);
	elseif (event == "MOUNT_JOURNAL_USABILITY_CHANGED") then
		if (not IsSubmerged()) then lastDryTime = GetTime() end
	elseif (event == "NEW_MOUNT_ADDED") then
		KhesycMountReload();
	end
end);

function KhesycMountBuild()
	KhesycMountSetup();
	KhesycMountSetBackdrops();
	KhesycMountSwitchLayout(km_indLayout);
end

function KhesycMountCheckVersion()
	if (km_version < addonVersion) then
		km_Tips:Show();
		km_version = addonVersion;
	end
end

function KhesycMountSetup()
	for i = 1, #ids do ids[i] = nil end
	for i = 1, #names do names[i] = nil end
	for i = 1, #nameKey do nameKey[i] = nil end
	maxInd = 1;

	local mounts = C_MountJournal.GetMountIDs();
	
	for i = 1, #mounts do
		local _, _, _, _, _, _, _, isFactionSpecific, faction, hideOnChar, isCollected, _ = C_MountJournal.GetMountInfoByID(mounts[i]);
		
		if
			isCollected
			and (not hideOnChar)
			and (
				(not isFactionSpecific)
				or (
					(faction == 1)
					--this returns the English name first and the local name second, so language is not an issue here
					and (UnitFactionGroup("player") == "Alliance")
				)
				or (
					(faction == 0)
					--this returns the English name first and the local name second, so language is not an issue here
					and (UnitFactionGroup("player") == "Horde")
				)
			) then
			
			if mounts[i] == 373 then
				hasVashjir = 1;
			end

			local mountName = GetMountName(mounts[i]);

			ids[maxInd] = mounts[i];
			names[maxInd] = mountName;
			nameKey[mountName] = mounts[i];

			maxInd = maxInd + 1;
		end
	end

	table.sort(names);
end

function KhesycMountReload()
	km_Settings:Hide();
	KhesycMountSetup();
end

function KhesycMountSetBackdrops()
	local backdrop = {
		bgFile = "Interface/BUTTONS/WHITE8X8",
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true,
		edgeSize = 16,
		tileSize = 16,
		insets = {
		left = 0,
		right = 0,
		top = 0,
		bottom = 0,
		},
	}
	km_Main:SetBackdrop(backdrop);
	km_Main:SetBackdropColor(0, 0, 0, .5);
	km_Settings:SetBackdrop(backdrop);
	km_Settings:SetBackdropColor(0, 0, 0, .5);
	km_Type:SetBackdrop(backdrop);
	km_Type:SetBackdropColor(0, 0, 0, .5);
	km_Categories:SetBackdrop(backdrop);
	km_Categories:SetBackdropColor(0, 0, 0, 1);
	km_DisplayMounts:SetBackdrop(backdrop);
	km_DisplayMounts:SetBackdropColor(0, 0, 0, .5);
	km_Tips:SetBackdrop(backdrop);
	km_Tips:SetBackdropColor(0, 0, 0, .75);
	km_DisplayMounts.ModelPreview:SetBackdrop(backdrop);
	km_DisplayMounts.ModelPreview:SetBackdropColor(0, 0, 0, .75);
end

function KhesycMountInit()
	if (km_ground == nil) then km_ground = {0} end
	if (km_flying == nil) then km_flying = {0} end
	if (km_dragon == nil) then km_dragon = {0} end
	if (km_gparty == nil) then km_gparty = {0} end
	if (km_fparty == nil) then km_fparty = {0} end
	if (km_vendor == nil) then km_vendor = {0} end
	if (km_swim == nil) then km_swim = {0} end
	if (km_qiraj == nil) then km_qiraj = {0} end
	if (km_maw == nil) then km_maw = {0} end
	if (km_version == nil) then km_version = "0.0.0" end
	if (km_mastLayout == nil) then km_mastLayout = "full" end
	if (km_indLayout == nil) then km_indLayout = km_mastLayout end
end

function KhesycMountSettingsOpen()
	if km_Settings:IsVisible() then
		km_Settings:Hide();
		km_Categories:Hide();
	else
		if (#ids < 1) then KhesycMountReload(); end
		km_Settings:Show();
		KhesycMountUpdateTypeOption(typeOption);
	end
end

function KhesycMountTypesOpen()
	if km_Categories:IsVisible() then
		km_Categories:Hide();
		km_DisplayMounts:Show();
	else
		km_Categories:Show();
		km_DisplayMounts:Hide();
	end
end

function KhesycMountUpdateTypeOption(newType)
	typeOption = newType;

	km_Type.Label:SetText(KhesycMountOptionTypeLongName(newType));

	if (effect ~= "add") then
		listInd = 1;
		KhesycMountSetMountList();
		KhesycMountUpdateMountList();
	end
	if (km_debug) then print("Updated Type: ".. typeOption); end
end

function KhesycMountOptionTypeLongName(shortName)
	if (shortName == "ground") then
		return "Ground";
	elseif (shortName == "flying") then
		return "Flying";
	elseif (shortName == "dragon") then
		return "Dragonriding";
	elseif (shortName == "gparty") then
		return "Group Ground";
	elseif (shortName == "fparty") then
		return "Group Flying";
	elseif (shortName == "vendor") then
		return "Vendor";
	elseif (shortName == "swim") then
		return "Swimming";
	elseif (shortName == "qiraj") then
		return "Ahn'Qiraj";
	elseif (shortName == "maw") then
		return "Maw";
	end
end

function KhesycMountSetMountList()
	if (typeOption == "ground") then
		km_list = km_ground;
	elseif (typeOption == "flying") then
		km_list = km_flying;
	elseif (typeOption == "gparty") then
		km_list = km_gparty;
	elseif (typeOption == "dragon") then
		km_list = km_dragon;
	elseif (typeOption == "fparty") then
		km_list = km_fparty;
	elseif (typeOption == "vendor") then
		km_list = km_vendor;
	elseif (typeOption == "swim") then
		km_list = km_swim;
	elseif (typeOption == "qiraj") then
		km_list = km_qiraj;
	elseif (typeOption == "maw") then
		km_list = km_maw;
	end
end

function KhesycMountUpdateMountList()
	if (effect == "add") then
		KhesycMountSetMount(km_DisplayMounts.pos_01, nameKey[km_list[listInd]]);
		KhesycMountSetMount(km_DisplayMounts.pos_02, nameKey[km_list[listInd + 1]]);
		KhesycMountSetMount(km_DisplayMounts.pos_03, nameKey[km_list[listInd + 2]]);
		KhesycMountSetMount(km_DisplayMounts.pos_04, nameKey[km_list[listInd + 3]]);
		KhesycMountSetMount(km_DisplayMounts.pos_05, nameKey[km_list[listInd + 4]]);
		KhesycMountSetMount(km_DisplayMounts.pos_06, nameKey[km_list[listInd + 5]]);
		KhesycMountSetMount(km_DisplayMounts.pos_07, nameKey[km_list[listInd + 6]]);
		KhesycMountSetMount(km_DisplayMounts.pos_08, nameKey[km_list[listInd + 7]]);
		KhesycMountSetMount(km_DisplayMounts.pos_09, nameKey[km_list[listInd + 8]]);
		KhesycMountSetMount(km_DisplayMounts.pos_10, nameKey[km_list[listInd + 9]]);
		KhesycMountSetMount(km_DisplayMounts.pos_11, nameKey[km_list[listInd + 10]]);
		KhesycMountSetMount(km_DisplayMounts.pos_12, nameKey[km_list[listInd + 11]]);
	else
		KhesycMountSetMount(km_DisplayMounts.pos_01, IsJointMount(km_list[listInd]));
		KhesycMountSetMount(km_DisplayMounts.pos_02, IsJointMount(km_list[listInd + 1]));
		KhesycMountSetMount(km_DisplayMounts.pos_03, IsJointMount(km_list[listInd + 2]));
		KhesycMountSetMount(km_DisplayMounts.pos_04, IsJointMount(km_list[listInd + 3]));
		KhesycMountSetMount(km_DisplayMounts.pos_05, IsJointMount(km_list[listInd + 4]));
		KhesycMountSetMount(km_DisplayMounts.pos_06, IsJointMount(km_list[listInd + 5]));
		KhesycMountSetMount(km_DisplayMounts.pos_07, IsJointMount(km_list[listInd + 6]));
		KhesycMountSetMount(km_DisplayMounts.pos_08, IsJointMount(km_list[listInd + 7]));
		KhesycMountSetMount(km_DisplayMounts.pos_09, IsJointMount(km_list[listInd + 8]));
		KhesycMountSetMount(km_DisplayMounts.pos_10, IsJointMount(km_list[listInd + 9]));
		KhesycMountSetMount(km_DisplayMounts.pos_11, IsJointMount(km_list[listInd + 10]));
		KhesycMountSetMount(km_DisplayMounts.pos_12, IsJointMount(km_list[listInd + 11]));
	end
end

function KhesycMountSetMount(asset, mountId)
	if (mountId == 0) then
		asset.Label:SetText("(none)");
		asset.Icon:Hide();
	elseif (mountId ~= nil) then
		local name = GetMountName(mountId);
		local _, _, icon = C_MountJournal.GetMountInfoByID(mountId);
		
		asset.Label:SetText(name);
		asset.Icon:SetNormalTexture(icon);
		asset.Icon:Show();
	else
		asset.Label:SetText("");
		asset.Icon:Hide();
	end
end

function KhesycMountScrollMountList(scrollAmt)
	listInd = listInd + scrollAmt;
	if (listInd < 1) then listInd = 1 end
	if (listInd > #km_list) then listInd = #km_list end

	KhesycMountUpdateMountList();
end

function KhesycMountChangeSelection(sel)
	if (sel == "add") then
		km_DisplayMounts.Add:Hide();
		km_DisplayMounts.Drop:Hide();
		km_DisplayMounts.Done:Show();

		effect = "add";
		km_list = names;
	elseif (sel == "drop") then
		km_DisplayMounts.Add:Hide();
		km_DisplayMounts.Drop:Hide();
		km_DisplayMounts.Done:Show();

		effect = "drop";
		KhesycMountSetMountList();
	else
		km_DisplayMounts.Add:Show();
		km_DisplayMounts.Drop:Show();
		km_DisplayMounts.Done:Hide();

		effect = "mount";
		KhesycMountSetMountList();
	end

	listInd = 1;
	KhesycMountUpdateMountList();
end

function KhesycMountRun(pos)
	if (effect == "add") then
		local addAt;
		if (nameKey[km_list[pos + listInd - 1]] ~= nil) then
			if (typeOption == "ground") then
				if (km_ground[1] == 0) then
					addAt = 1;
				else
					addAt = #km_ground + 1;
				end
				km_ground[addAt] = nameKey[km_list[pos + listInd - 1]];
			elseif (typeOption == "flying") then
				if (km_flying[1] == 0) then
					addAt = 1;
				else
					addAt = #km_flying + 1;
				end
				km_flying[addAt] = nameKey[km_list[pos + listInd - 1]];
			elseif (typeOption == "dragon") then
				if (km_dragon[1] == 0) then
					addAt = 1;
				else
					addAt = #km_dragon + 1;
				end
				km_dragon[addAt] = nameKey[km_list[pos + listInd - 1]];
			elseif (typeOption == "gparty") then
				if (km_gparty[1] == 0) then
					addAt = 1;
				else
					addAt = #km_gparty + 1;
				end
				km_gparty[addAt] = nameKey[km_list[pos + listInd - 1]];
			elseif (typeOption == "fparty") then
				if (km_fparty[1] == 0) then
					addAt = 1;
				else
					addAt = #km_fparty + 1;
				end
				km_fparty[addAt] = nameKey[km_list[pos + listInd - 1]];
			elseif (typeOption == "vendor") then
				if (km_vendor[1] == 0) then
					addAt = 1;
				else
					addAt = #km_vendor + 1;
				end
				km_vendor[addAt] = nameKey[km_list[pos + listInd - 1]];
			elseif (typeOption == "swim") then
				if (km_swim[1] == 0) then
					addAt = 1;
				else
					addAt = #km_swim + 1;
				end
				km_swim[addAt] = nameKey[km_list[pos + listInd - 1]];
			elseif (typeOption == "qiraj") then
				if (km_qiraj[1] == 0) then
					addAt = 1;
				else
					addAt = #km_qiraj + 1;
				end
				km_qiraj[addAt] = nameKey[km_list[pos + listInd - 1]];
			elseif (typeOption == "maw") then
				if (km_maw[1] == 0) then
					addAt = 1;
				else
					addAt = #km_maw + 1;
				end
				km_maw[addAt] = nameKey[km_list[pos + listInd - 1]];
			end
			print('Mount "' .. km_list[pos + listInd - 1] .. '" added to "' .. KhesycMountOptionTypeLongName(typeOption) .. '"');
		end
	elseif (effect == "drop") then
		if (km_list[pos + listInd - 1] ~= nil) then
			table.remove(km_list, pos + listInd -1);
			
			KhesycMountSetMountList();
			KhesycMountUpdateMountList();
		end
	else
		if (km_list[pos + listInd - 1] ~= nil) then
			SummonMount(IsJointMount(km_list[pos + listInd - 1]));
			KhesycMountSettingsOpen();
		end
	end
end

function GetMountName(id)
	if (id == nil) then
		return "";
	elseif (id == 0) then
		return "(none)";
	--elseif (id == 240 or id == 275) then
		--Mekgineer's Chopper and Mechano-Hog
		--return GetJointMountName(275, 240);
	else
		local mountName = C_MountJournal.GetMountInfoByID(id);
		return mountName;
	end
end

function IsJointMount(id)
	local r;

	if (id == 240 or id == 275) then
		--Mekgineer's Chopper (Alliance) = 275
		--Mechano-Hog (Horde) = 240
		r = ResolveJointMount(275, 240);
	elseif (id == 1246 or id == 1245) then
		--Ironclad Frostpaw (Alliance) = 1246
		--Bloodflank Charger (Horde) = 1245
		r = ResolveJointMount(1246, 1245);
	else
		r = id;
	end

	return r;
end

function ResolveJointMount(alliance, horde)
	if (UnitFactionGroup("player") == "Alliance") then
		return alliance;
	else
		return horde;
	end
end

function GetMountIcon(id)
	if (id == nil) then
		return "";
	else
		local _, _, icon = C_MountJournal.GetMountInfoByID(id);
		return icon;
	end
end

function SummonMount(id)
	if not IsFlying() then
		if IsMounted() then
			if (km_debug) then print("Player is mounted. Attempt dismount."); end
			Dismount();
		else
			if (km_debug) then print("Mount resolved. Attempt to summon: " .. id .. " - " .. GetMountName(id)); end
			C_MountJournal.SummonByID(id);
		end
	else
		if (km_debug) then print("Player is flying. Unable to safely dismount"); end
	end
end

function ResolveArea()
	if IsInMaw() and (#km_maw > 0) and (km_maw[1] ~= 0) then
		if (km_debug) then print("Resolve Mount: Is In The Maw"); end
		return km_maw[math.random(1, #km_maw)];
	elseif IsInAhnQiraj() and (#km_qiraj > 0) and (km_qiraj[1] ~= 0) then
		if (km_debug) then print("Resolve Ground: Is In Ahn'Qiraj"); end
		return km_qiraj[math.random(1, #km_qiraj)];
	elseif IsDragonriding() and (#km_dragon > 0) and (km_dragon[1] ~= 0) then
		if (km_debug) then print("Resolve Mount: Is Dragonriding Area"); end
		return km_dragon[math.random(1, #km_dragon)];
	else
		return 0;
	end
end

function SummonChauffer()
	SummonMount(ResolveChauffer());
end

function ResolveChauffer()
	if (UnitFactionGroup("player") == "Alliance") then
		if (km_debug) then print("Resolve Chauffer: Alliance"); end
		return 679;
	elseif (UnitFactionGroup("player") == "Horde") then
		if (km_debug) then print("Resolve Chauffer: Horde"); end
		return 678;
	end
end

function SummonVendor()
	SummonMount(ResolveVendor());
end

function ResolveVendor()
	if (#km_vendor == 0) then
		if (km_debug) then print("Resolve Vendor: Summon Favorite"); end
		return 0;
	else
		if (km_debug) then print("Resolve Vendor: Summon Selected"); end
		return km_vendor[math.random(1, #km_vendor)];
	end
end

function SummonSwimming()
	SummonMount(ResolveSwimming());
end

function ResolveSwimming()
	--ResetPlayerPitch()
	if IsInVashjir() and (hasVashjir == 1) then
		if (km_debug) then print("Resolve Swimming: Is In Vashj'ir"); end
		--Vashj'ir Seahorse
		return 373;
	else
		--swimming
		if (#km_swim == 0) then
			if (km_debug) then print("Resolve Swimming: Summon Favorite"); end
			return 0;
		else
			if (km_debug) then print("Resolve Swimming: Summon Selected"); end
			return km_swim[math.random(1, #km_swim)];
		end
	end
end

function UseSwimming()
	return (GetTime() - lastDryTime >= 1);
end

function IsInVashjir()
	return (
		GetRealZoneText() == "Shimmering Expanse"
		or GetRealZoneText() == "Kelp'thar Forest"
		or GetRealZoneText() == "Abyssal Depths"
		or GetRealZoneText() == "Vashj'ir"
	);
end

function IsInAhnQiraj()
	return GetRealZoneText() == "Ahn'Qiraj"
end

function IsInMaw()
	return GetRealZoneText() == "The Maw"
end

function IsDragonriding()
	return IsUsableSpell(368896);
end

function SummonFlying()
	SummonMount(ResolveFlying());
end

function ResolveFlying()
	if (#km_flying == 0) then
		if (km_debug) then print("Resolve Flying: Summon Favorite"); end
		return 0;
	else
		if (km_debug) then print("Resolve Flying: Summon Selected"); end
		return km_flying[math.random(1, #km_flying)];
	end
end

function SummonFParty()
	SummonMount(ResolveFParty());
end

function ResolveFParty()
	if (#km_fparty == 0) then
		if (km_debug) then print("Resolve Flying - Party: Summon Favorite"); end
		return 0;
	else
		if (km_debug) then print("Resolve Flying - Party: Summon Selected"); end
		return km_fparty[math.random(1, #km_fparty)];
	end
end

function SummonGround()
	SummonMount(ResolveGround());
end

function ResolveGround()
	if (#km_ground == 0) then
		if (km_debug) then print("Resolve Ground: Summon Favorite"); end
		return 0;
	else
		if (km_debug) then print("Resolve Ground: Summon Selected"); end
		return km_ground[math.random(1, #km_ground)];
	end
end

function SummonGParty()
	SummonMount(ResolveGParty());
end

function ResolveGParty()
	local r;

	if (#km_gparty == 0) then
		r = 0;
	else
		r = km_gparty[math.random(1, #km_gparty)];
	end

	--TODO: Update these mounts to use the joint mount feature
	--this returns the English name first and the local name second, so language is not an issue here
	if (UnitFactionGroup("player") == "Alliance") and (r == 240) then
		r = 275;
	--this returns the English name first and the local name second, so language is not an issue here
	elseif (UnitFactionGroup("player") == "Horde") and (r == 275) then
		r = 240;
	end

	return r;
end

function ToonCanFlyArea()
	return (ToonCanFly() and IsFlyableArea());
end

function ToonCanFly()
	if (IsSpellKnown(34090) or IsSpellKnown(90265)) then
		return true;
	elseif (UnitLevel("player") >= 27 and not IsSpellKnown(33388) and not IsSpellKnown(33391)) then
		--this logic is a "trap" to detect toons that have the Legacy - Artisan Riding skill
		--if they have no riding skill at all and their level is greater than or equal to 27 (70 pre-Shadowlands)
		--  then we can assume they have the Legacy - Artisan Riding skill
		return true;
	else
		return false;
	end
end

function ToonCanMount()
	if (IsSpellKnown(33388) or IsSpellKnown(33391) or IsSpellKnown(34090) or IsSpellKnown(90265)) then
		return true;
	elseif (UnitLevel("player") >= 27) then
		--this logic is a "trap" to detect toons that have the Legact - Artisan Riding skill
		--if they have no riding skill at all and their level is greater than or equal to 27 (70 pre-Shadowlands)
		--  then we can assume they have the Legacy - Artisan Riding skill
		return true;
	else
		return false;
	end
end

function KhesycMountSummon()
	if not IsFlying() then
		--special mounted commands
		if IsMounted() then
			if IsControlKeyDown() then
				EjectPassengerFromSeat(1);
				EjectPassengerFromSeat(2);

			elseif IsShiftKeyDown() then
				DoEmote(EMOTE171_TOKEN);

			elseif pitchLock == 1 then
				ResetPlayerPitch();

			else
				Dismount();

			end
		--summon mount commands
		else
			if not ToonCanMount() then
				--no riding skill, summon chauffered mount by faction
				SummonChauffer();

			elseif IsShiftKeyDown() then
				--vendor mount
				SummonVendor();

			elseif IsControlKeyDown() then
				--summon random favorite mount
				SummonMount(0);

			elseif IsSubmerged() and UseSwimming() then
				SummonSwimming();

			--elseif IsAltKeyDown() then
			--	ResetPlayerPitch()

			else
				if (ResolveArea() ~= 0) then
					SummonMount(ResolveArea());
				elseif ToonCanFlyArea() then
					if (not UnitInRaid("player") and UnitInParty("player") and (GetNumGroupMembers() > 1)) then
						--flying party
						SummonFParty();
					else
						--flying
						SummonFlying();
					end
				else
					if (not UnitInRaid("player") and UnitInParty("player") and (GetNumGroupMembers() > 1)) then
						--ground party
						SummonGParty();
					else
						--ground
						SummonGround();
					end
				end
			end
		end
	else
		--ResetPlayerPitch()
	end

	if (km_debug == true) then print(" "); end
end

local pitchLock = 0;
function ResetPlayerPitch()
	--print("Attempting to reset pitch...");\
	--TODO: make the movement angle match the camera angle
	--NOTE: if I can force the movement angle to match the camera angle then
		--I can remove all the pitchLock variable references and the toggled
		--pitch lock state by just setting the pitchlock, matching the movement
		--angle, then restoring the pitchlock again.
		--Currently this tactic isn't allowed by Blizz.
	if (pitchLock == 0) then
		--this command changes the view angle, but not the movement angle
		print("Khes'yc Mount - Pitch Locked!");
		ConsoleExec("pitchlimit 0");
		pitchLock = 1;
	else
		--this restores the view angle to default
		print("Khes'yc Mount - Flying Free!");
		ConsoleExec("pitchlimit 90");
		pitchLock = 0;
	end
end

function HandleSpeedEvent()
	local currentSpeed = GetUnitSpeed("player");
	currentSpeed = floor((currentSpeed / 7 * 100) + 0.1);
	km_Main.Speed:SetText(tostring(currentSpeed) .. "%");
end

function KhesycMountSetTooltip(pos)
	local mountId = 0;

	if (effect == "add") then
		mountId = nameKey[km_list[pos + listInd - 1]];
	else
		mountId = km_list[pos + listInd - 1];
	end

	if (mountId ~= nil) then
		local _, spellId = C_MountJournal.GetMountInfoByID(mountId);
		local displayId, description, source = C_MountJournal.GetMountInfoExtraByID(mountId);

		if (displayId ~= nil) then
			km_DisplayMounts.ModelPreview:Show();
			km_DisplayMounts.ModelPreview.Model:SetDisplayInfo(displayId);
            km_DisplayMounts.ModelPreview.Model:SetAnimation(618, -1);
			km_DisplayMounts.ModelPreview.Model:SetRotation(-MODELFRAME_DEFAULT_ROTATION);
		end

		if (spellId ~= nil) then
			GameTooltip:SetOwner(km_DisplayMounts.ModelPreview.Model, "ANCHOR_BOTTOM");
			GameTooltip:ClearLines();
			GameTooltip:SetMountBySpellID(spellId);
			GameTooltip:Show();
		end
	end
end

function KhesycMountResetTooltip()
	km_DisplayMounts.ModelPreview:Hide();
	GameTooltip:Hide();
end

function KhesycMountSwitchLayout(layout)
	if (layout == "full") then
		km_Main:SetWidth(215);
		km_Pitch:SetPoint("LEFT", km_Main, "LEFT", 160, 0);
		km_Mount:Show();
		km_SettingsOpen:Show();

		km_indLayout = "full";
	elseif (layout == "min") then
		km_Mount:Hide();
		km_SettingsOpen:Hide();
		km_Pitch:SetPoint("LEFT", km_Main, "LEFT", 50, 0);
		km_Main:SetWidth(80);

		km_indLayout = "min";
	else
		print('Option not recognized. Please try "full" or "min".');
	end

	km_btnLayout:SetText(KhesycMountGetLayout());
	km_mastLayout = km_indLayout;
end

function KhesycMountGetLayout()
	if (km_indLayout == "full") then
		return "Full";
	elseif (km_indLayout == "min") then
		return "Minimal";
	end
end